@@ -34,7 +34,7 @@
 #include "vstr.h"
 #include "config.h"
 
-static char gp_title[] = "dvdspanky 1.0.5 - (C) 2007 Jeffrey Grembecki\n\n";
+static char gp_title[] = "dvdspanky 1.0.6 - (C) 2007 Jeffrey Grembecki\n\n";
 static char gp_licence[] =
 "This program is free software; you can redistribute it and/or modify\n"
 "it under the terms of the GNU General Public License as published by\n"
@@ -131,6 +131,7 @@ static pcre *gp_idregex = NULL;
 static pcre *gp_transregex = NULL;
 static pcre *gp_tcvideoregex = NULL;
 static pcre *gp_tcaudioregex = NULL;
+static pcre *gp_warnerrregex = NULL;
 
 #define ML_ERROR 4
 #define ML_WARNING 3
@@ -141,101 +142,46 @@ static pcre *gp_tcaudioregex = NULL;
 static int g_msglevel = ML_INFO;
 
 static FILE *gp_logfile = NULL;
-static vstr *gp_str = NULL;
+static char gp_printstr[2048];
 
-static void print(int msglevel, const char *p_format, ...)
+static void print(int msglevel, const char *p_prefix, const char *p_format, ...)
 {
-	char *p_dyn;
-	int length;
 	va_list argp;
 
-	p_dyn = NULL;
-
 	va_start(argp, p_format);
-	length = vsnprintf(NULL, 0, p_format, argp);
+	vsnprintf(gp_printstr, sizeof(gp_printstr), p_format, argp);
+	gp_printstr[sizeof(gp_printstr) - 1] = 0;
 	va_end(argp);
 
-	if(length && (p_dyn = malloc(length + 1)))
+	if(msglevel >= g_msglevel)
+		fprintf(stderr, "%s\n", gp_printstr);
+
+	if(gp_logfile)
 	{
-		va_start(argp, p_format);
-		vsprintf(p_dyn, p_format, argp);
-		va_end(argp);
-
-		if(msglevel >= g_msglevel)
-			fprintf(stderr, "%s", p_dyn);
-
-		if(gp_logfile)
-		{
-			vs_print(gp_str, " dvdspanky | %s", p_dyn);
-			fwrite(gp_str->s, 1, gp_str->len, gp_logfile);
-			fflush(gp_logfile);
-		}
-
-		free(p_dyn);
-	}
-}
-
-static void pprint(int msglevel, const char *p_prefix, const char *p_format, ...)
-{
-	char *p_dyn;
-	int length;
-	va_list argp;
-
-	p_dyn = NULL;
-
-	va_start(argp, p_format);
-	length = vsnprintf(NULL, 0, p_format, argp);
-	va_end(argp);
-
-	if(length && (p_dyn = malloc(length + 1)))
-	{
-		va_start(argp, p_format);
-		vsprintf(p_dyn, p_format, argp);
-		va_end(argp);
-
-		if(msglevel >= g_msglevel)
-			fprintf(stderr, "%s", p_dyn);
-
-		if(gp_logfile)
-		{
-			vs_print(gp_str, "%10s | %s", p_prefix, p_dyn);
-			fwrite(gp_str->s, 1, gp_str->len, gp_logfile);
-			fflush(gp_logfile);
-		}
-
-		free(p_dyn);
+		if(p_prefix)
+			fprintf(gp_logfile, "%10s | %s\n", p_prefix, gp_printstr);
+		else
+			fprintf(gp_logfile, " dvdspanky | %s\n", gp_printstr);
+		fflush(gp_logfile);
 	}
 }
 
 static void printerror(const char *p_format, ...)
 {
-	char *p_dyn;
-	int length;
 	va_list argp;
 
-	p_dyn = NULL;
-
 	va_start(argp, p_format);
-	length = vsnprintf(NULL, 0, p_format, argp);
+	vsnprintf(gp_printstr, sizeof(gp_printstr), p_format, argp);
+	gp_printstr[sizeof(gp_printstr) - 1] = 0;
 	va_end(argp);
 
-	if(length && (p_dyn = malloc(length + 1)))
+	if(ML_ERROR >= g_msglevel)
+		fprintf(stderr, "error: %s\n", gp_printstr);
+
+	if(gp_logfile)
 	{
-		va_start(argp, p_format);
-		vsprintf(p_dyn, p_format, argp);
-		va_end(argp);
-
-		if(ML_ERROR >= g_msglevel)
-			fprintf(stderr, "%s", p_dyn);
-
-		if(gp_logfile)
-		{
-			vs_print(gp_str, " dvdspanky | %s", p_dyn);
-			fwrite(gp_str->s, 1, gp_str->len, gp_logfile);
-			fflush(gp_logfile);
-		}
-
-		free(p_dyn);
+		fprintf(gp_logfile, " dvdspanky | error: %s\n", gp_printstr);
+		fflush(gp_logfile);
 	}
 }
 
@@ -303,14 +249,14 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 			" --encode_fields %c", 	p_out->vrate, p_in->fps, p_out->fps, asr, asr,
 			(p_out->p_vcodec[0] == 'P') ? 't' : 'b');
 	}
-	if(options & SPANK_DOUBLEFPS)
-		vs_cat(p_video, " -J doublefps");
-	if(options & SPANK_NEWFPS)
-		vs_cat(p_video, " -J modfps=clonetype=1");
-	if(options & SPANK_FIXNTSC)
-		vs_cat(p_video, " -M 2 -J ivtc,32detect=force_mode=5,decimate --hard_fps");
 
 	/* video extra */
+	if(options & SPANK_DOUBLEFPS)
+		vs_cat(p_videoextra, " -J doublefps");
+	if(options & SPANK_NEWFPS)
+		vs_cat(p_videoextra, " -J modfps=clonetype=1");
+	if(options & SPANK_FIXNTSC)
+		vs_cat(p_videoextra, " -M 2 -J ivtc,32detect=force_mode=5,decimate --hard_fps");
 	if(p_out->vcrop || p_out->hcrop)
 		vs_cat(p_videoextra, " -j %d,%d,%d,%d",
 			p_out->vcrop, p_out->hcrop, p_out->vcrop, p_out->hcrop);
@@ -333,10 +279,10 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		struct dirent **pp_dirent;
 		int count;
 
-		print(ML_INFO, "generating previews..\n"); /* generate previews */
-		if(forkitf(NULL, NULL, TRANSCODE_BIN, "%s %s %s -y jpg,null -F 100 -Z %dx%d"
+		print(ML_INFO, NULL, "generating previews.."); /* generate previews */
+		if(forkitf(NULL, "transcode", TRANSCODE_BIN, "%s %s -y jpg,null -F 100 -Z %dx%d"
 			" -c 300-301,600-601,900-901,1200-1201,1500-1501 -o pre",
-			p_generic->s, p_video->s, p_videoextra->s,
+			p_generic->s, p_video->s,
 			(int)((double)p_out->height * p_out->aspect), p_out->height, p_out->p_filename))
 		{
 			printerror("failed generating previews");
@@ -349,7 +295,7 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		{
 			int found = count;
 
-			print(ML_INFO, "displaying previews..\n");
+			print(ML_INFO, NULL, "displaying previews..");
 
 			vs_print(p_final, "feh");
 			while(count--)
@@ -357,7 +303,7 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 				if(!fnmatch("pre*.jpg", pp_dirent[count]->d_name, 0))
 					vs_cat(p_final, " %s", pp_dirent[count]->d_name);
 			}
-			forkit(NULL, NULL, FEH_BIN, p_final->s);
+			forkit(NULL, "feh", FEH_BIN, p_final->s);
 
 			count = found;
 			while(count--)
@@ -375,19 +321,19 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 	/* auto volume only pass */
 	if(options & SPANK_AVPASS)
 	{
-		print(ML_INFO, "scanning volume..\n");
+		print(ML_INFO, NULL, "scanning volume..");
 		vs_print(p_final, "%s-tmp.ac3", p_out->p_filename);
 		if(forkitf(cb_transcode, (void*)p_out, TRANSCODE_BIN,
 			"%s %s -y null,ac3 -m %s-tmp -J astat=%s.norm",
 			p_generic->s, p_audio->s, p_out->p_filename, p_out->p_filename))
 		{
-			print(ML_INFO, "\n");
+			print(ML_INFO, NULL, "");
 			printerror("error scanning volume");
 			unlink(p_final->s);
 			freestrings();
 			return 1;
 		}
-		print(ML_INFO, "\n");
+		print(ML_INFO, NULL, "");
 		unlink(p_final->s);
 		return 0;
 	}
@@ -395,7 +341,7 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 	/* execute 1st pass */
 	if(options & SPANK_FIRST && !(options & SPANK_MULTIPLEX))
 	{
-		print(ML_INFO, "starting 1st VBR pass..\n");
+		print(ML_INFO, NULL, "starting 1st VBR pass..");
 
 		vs_print(p_final, "%s %s %s", p_generic->s, p_video->s, p_audio->s);
 		if(options & SPANK_AVOLUME)
@@ -409,7 +355,7 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		{
 			vs_print(p_final, "%s.tmp", p_out->p_filename);
 			unlink(p_final->s);
-			print(ML_INFO, "\n");
+			print(ML_INFO, NULL, "");
 			printerror("transcode failed");
 			freestrings();
 			return 1;
@@ -417,7 +363,7 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		vs_print(p_final, "%s.tmp", p_out->p_filename);
 		unlink(p_final->s);
 
-		print(ML_INFO, "\n");
+		print(ML_INFO, NULL, "");
 	}
 
 	/* execute 2nd pass */
@@ -465,7 +411,7 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		}
 
 		/* transcode */
-		print(ML_INFO, "starting 2nd VBR pass..\n");
+		print(ML_INFO, NULL, "starting 2nd VBR pass..");
 		vs_print(p_final,
 			"%s %s %s %s -Z %dx%d -y ffmpeg -F mpeg2video -R 2,%s.pass1 -o %s -m %s.ac3",
 			p_generic->s, p_audio->s, p_video->s, p_videoextra->s,
@@ -474,12 +420,12 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 			vs_cat(p_final, " -s %f", p_out->volume);
 		if(forkitf(cb_transcode, (void*)p_out, TRANSCODE_BIN, p_final->s))
 		{
-			print(ML_INFO, "\n");
+			print(ML_INFO, NULL, "");
 			printerror("transcode failed");
 			freestrings();
 			return 1;
 		}
-		print(ML_INFO, "\n");
+		print(ML_INFO, NULL, "");
 	}
 
 	/* CBR encoding */
@@ -491,15 +437,15 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 			char p_buffer[320];
 			FILE *p_file;
 
-			print(ML_INFO, "scanning volume..\n");
+			print(ML_INFO, NULL, "scanning volume..");
 			if(forkitf(cb_transcode, (void*)p_out, TRANSCODE_BIN,
 				"%s %s -y null,ac3 -m %s-tmp -J astat=%s.norm",
 				p_generic->s, p_audio->s, p_out->p_filename, p_out->p_filename))
 			{
-				print(ML_INFO, "\n");
+				print(ML_INFO, NULL, "");
 				printerror("error scanning volume");
 			}
-			print(ML_INFO, "\n");
+			print(ML_INFO, NULL, "");
 
 			vs_print(p_final, "%s-tmp.ac3", p_out->p_filename);
 			unlink(p_final->s);
@@ -524,7 +470,7 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		}
 
 		/* execute */
-		print(ML_INFO, "starting CBR encode..\n");
+		print(ML_INFO, NULL, "starting CBR encode..");
 		vs_print(p_final, "%s %s %s %s -Z %dx%d -y ffmpeg -F mpeg2video -o %s -m %s.ac3",
 			p_generic->s, p_video->s, p_videoextra->s, p_audio->s,
 			p_out->width, p_out->height, p_out->p_filename, p_out->p_filename);
@@ -532,26 +478,26 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 			vs_cat(p_final, " -s %f", p_out->volume);
 		if(forkit(cb_transcode, (void*)p_out, TRANSCODE_BIN, p_final->s))
 		{
-			print(ML_INFO, "\n");
+			print(ML_INFO, NULL, "");
 			printerror("transcode failed");
 			freestrings();
 			return 2;
 		}
-		print(ML_INFO, "\n");
+		print(ML_INFO, NULL, "");
 	}
 
 	/* multiplex */
 	if((options & SPANK_SECOND) || !(options & (SPANK_FIRST | SPANK_SECOND)) ||
 		(options & SPANK_MULTIPLEX))
 	{
-		print(ML_INFO, "multiplexing..\n");
+		print(ML_INFO, NULL, "multiplexing..");
 		if((options & SPANK_SECOND))
 		{
-			if(forkitf(NULL, "mplex |", MPLEX_BIN,
+			if(forkitf(NULL, "mplex", MPLEX_BIN,
 				"mplex -V -f 8 -o %s.mpg %s.m2v %s.ac3",
 				p_out->p_filename, p_out->p_filename, p_out->p_filename))
 				{
-					print(ML_INFO, "\n");
+					print(ML_INFO, NULL, "");
 					printerror("mplex failed");
 					freestrings();
 					return 3;
@@ -559,18 +505,18 @@ int encode(video_t *p_in, video_t *p_out, int options, const char *p_filters)
 		}
 		else
 		{
-			if(forkitf(NULL, "mplex |", MPLEX_BIN,
+			if(forkitf(NULL, "mplex", MPLEX_BIN,
 				"mplex -f 8 -o %s.mpg %s.m2v %s.ac3",
 				p_out->p_filename, p_out->p_filename, p_out->p_filename))
 			{
-				print(ML_INFO, "\n");
+				print(ML_INFO, NULL, "");
 				printerror("mplex failed");
 				freestrings();
 				return 3;
 			}
 		}
 
-		print(ML_INFO, "\n");
+		print(ML_INFO, NULL, "");
 		vs_print(p_final, "%s.m2v", p_out->p_filename);
 		unlink(p_final->s);
 		vs_print(p_final, "%s.ac3", p_out->p_filename);
@@ -601,7 +547,10 @@ int cb_tcprobe(const char *p_string, void *p_options)
 		return 0;
 	}
 
-	pprint(ML_VERBOSE, "tcprobe", "%s\n", p_string);
+	if(pcre_exec(gp_idregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
+		print(ML_WARNING, "tcprobe", "%s", p_string);
+	else
+		print(ML_VERBOSE, "tcprobe", "%s", p_string);
 
 	/* video regex */
 	length = strlen(p_string);
@@ -721,7 +670,10 @@ int cb_transcode(const char *p_string, void *p_options)
 	}
 	else
 	{
-		pprint(ML_VERBOSE, "transcode", "%s\n", p_string);
+		if(pcre_exec(gp_idregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
+			print(ML_WARNING, "transcode", "%s", p_string);
+		else
+			print(ML_VERBOSE, "transcode", "%s", p_string);
 	}
 
 	return 0;
@@ -743,7 +695,10 @@ int cb_mplayer(const char *p_string, void *p_options)
 	/* option is output to a video_t */
 	p_video = (video_t*)p_options;
 
-	pprint(ML_VERBOSE, "mplayer", "%s\n", p_string);
+	if(pcre_exec(gp_idregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
+		print(ML_WARNING, "mplayer", "%s", p_string);
+	else
+		print(ML_VERBOSE, "mplayer", "%s", p_string);
 
 	/* examine the string, return if nothing */
 	numstrings = pcre_exec(gp_idregex, NULL, p_string, strlen(p_string), 0, 0, p_vector, 20);
@@ -792,13 +747,15 @@ int cb_mplayer(const char *p_string, void *p_options)
 /* generic callback */
 int cb_default(const char *p_string, void *p_options)
 {
-
 	/* initialisation command */
 	if(!p_string)
 		return 0;
 
-	/* output le shiz */
-	pprint(ML_VERBOSE, (char*)p_options, "%s\n", p_string);
+	/* check line against error|warning filter */
+	if(pcre_exec(gp_idregex, NULL, p_string, strlen(p_string), 0, 0, NULL, 0) >= 1)
+		print(ML_WARNING, (char*)p_options, "%s", p_string);
+	else
+		print(ML_VERBOSE, (char*)p_options, "%s", p_string);
 
 	return 0;
 }
@@ -823,7 +780,7 @@ int forkit(forkitcallback callback, void *p_options, const char *p_path, const c
 	/* create parameter index */
 	strncpy(p_cmdline, p_cmd, cmdlength + 1);
 	pos = start = 0;
-	print(ML_VERBOSE, "fork %s, %s\n", p_path,p_cmd);
+	print(ML_VERBOSE, NULL, "fork %s, %s", p_path, p_cmd);
 
 	for(i = 0; i < sizeof(pp_cmdopts) / sizeof(char**); i++)
 	{
@@ -956,11 +913,6 @@ int main(int argc, char **argv)
 
 	/* setup */
 	p_filters[0] = 0;
-	if(!(gp_str = vs_new()))
-	{
-		fprintf(stderr, "unable to allocate gp_str\n");
-		exit(1);
-	}
 
 	/* clear video data */
 	memset(&invid, '\0', sizeof(video_t));
@@ -970,7 +922,7 @@ int main(int argc, char **argv)
 	atexit(cleanup);
 
 	/* show title */
-	print(ML_INFO, gp_title);
+	printf("%s", gp_title);
 
 	/* compile regex */
 	if(!(gp_transregex = pcre_compile(
@@ -997,6 +949,11 @@ int main(int argc, char **argv)
 		printerror("error: gp_tcaudioregex compile failed - %s", p_error);
 		exit(1);
 	}
+	if(!(gp_warnerrregex = pcre_compile("^.*(?:warning22|error22).*$", PCRE_CASELESS, &p_error, &erroffset, NULL)))
+	{
+		printerror("error: gp_warnerrregex compile failed - %s", p_error);
+		exit(1);
+	}
 
 	/* parse options */
 	while( (opt = getopt_long(argc, argv, gp_shortopts, gp_longopts, NULL)) != -1 )
@@ -1052,7 +1009,7 @@ int main(int argc, char **argv)
 				outvid.gamma = atof(optarg);
 				break;
 			case 'h':
-				printf(gp_help, argv[0]);
+				fprintf(stderr, gp_help, argv[0]);
 				exit(0);
 				break;
 			case 'H':
@@ -1065,7 +1022,7 @@ int main(int argc, char **argv)
 				options |= SPANK_NONICE;
 				break;
 			case 'L':
-				printf(gp_licence);
+				fprintf(stderr, gp_licence);
 				exit(0);
 				break;
 			case 'm':
@@ -1385,29 +1342,29 @@ int main(int argc, char **argv)
 	outvid.bytes = (vrate + vrate / 20 + arate + arate / 20) / 8 * invid.length * 1000;
 
 	/* print source and dest video encoding info */
-	print(ML_INFO, " source | %s\n", invid.p_filename);
-	print(ML_INFO, "  video | %s, %dx%d, %d kbps, %.3f fps\n",
+	print(ML_INFO, NULL, " source | %s", invid.p_filename);
+	print(ML_INFO, NULL,  "  video | %s, %dx%d, %d kbps, %.3f fps",
 		invid.p_vcodec, invid.width, invid.height, invid.vrate, invid.fps);
-	print(ML_INFO, "        | %.4f aspect, %d frames, %d:%02d:%02d\n",
+	print(ML_INFO, NULL,  "        | %.4f aspect, %d frames, %d:%02d:%02d",
 		invid.aspect, invid.frames,	invid.hours, invid.minutes, invid.seconds);
-	print(ML_INFO, "  audio | %s, %d kbps, %d Hz, %d bit, %d channel\n",
+	print(ML_INFO, NULL,  "  audio | %s, %d kbps, %d Hz, %d bit, %d channel",
 		invid.p_acodec, invid.arate, invid.hz, invid.abits, invid.achans);
-	print(ML_INFO, "   file | %d KB\n", invid.bytes / 1000);
+	print(ML_INFO, NULL,  "   file | %d KB", invid.bytes / 1000);
 	if(options & SPANK_MPLAYER)
-		print(ML_INFO, "   info | using mplayer decoder\n");
-	print(ML_INFO, "\n");
+		print(ML_INFO, NULL, "   info | using mplayer decoder");
+	print(ML_INFO, NULL,  "");
 
-	print(ML_INFO, "   dest | %s.mpg\n", outvid.p_filename);
-	print(ML_INFO, "  video | %s, %dx%d, %d kbps, %.3f fps\n",
+	print(ML_INFO, NULL,  "   dest | %s.mpg", outvid.p_filename);
+	print(ML_INFO, NULL,  "  video | %s, %dx%d, %d kbps, %.3f fps",
 		outvid.p_vcodec, outvid.width, outvid.height, outvid.vrate, outvid.fps);
-	print(ML_INFO, "        | %.4f aspect, %d frames, %d:%02d:%02d\n",
+	print(ML_INFO, NULL,  "        | %.4f aspect, %d frames, %d:%02d:%02d",
 		outvid.aspect, outvid.frames, outvid.hours, outvid.minutes, outvid.seconds);
-	print(ML_INFO, "        | %d(v):%d(h) crop, %s\n",
+	print(ML_INFO, NULL,  "        | %d(v):%d(h) crop, %s",
 		outvid.vcrop, outvid.hcrop, (options & SPANK_MULTI) ? "VBR" : "CBR");
-	print(ML_INFO, "  audio | %s, %d kbps, %d Hz, %d bit, %d channel\n",
+	print(ML_INFO, NULL,  "  audio | %s, %d kbps, %d Hz, %d bit, %d channel",
 		outvid.p_acodec, outvid.arate, outvid.hz, outvid.abits, outvid.achans);
-	print(ML_INFO, "   file | ~%d KB\n", outvid.bytes / 1000);
-	print(ML_INFO, "\n");
+	print(ML_INFO, NULL,  "   file | ~%d KB", outvid.bytes / 1000);
+	print(ML_INFO, NULL,  "");
 
 	exit(encode(&invid, &outvid, options, p_filters));
 }
